/* $Id: icb.h,v 1.22 2000/12/15 04:29:18 emf Exp $ */

#include "config.h"

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#include <tcl.h>

#include "protocol.h"

/* generic message buffer size */
/* this needs to be bigger than USER_BUF_SIZE defined in ../murgil/ipcf.h */
#define MESSAGE_BUF_SIZE 2048 


/* command usage information */

typedef struct Usage {
	char name[9];
	char type;
	char args[128];
	char usage[128];
	struct Usage *prev, *next;
} USAGE;

/* special global variables */

typedef struct {
	char cmdchar;		/* character to prefix a command */
	int verifyquit;		/* verify a quit always */
	int pauseonshell;	/* make user hit ret on shell return */
	int interactive;	/* nonzero if interactive session */
	int pagesize;		/* lines per page */
	int phistory;		/*  */
	int bufferlines;	/* lines in the review buffer */
	int groupblanks;	/* print a blank before each group */
	char *logfile;		/* name of session log file */
	int restricted;		/* nonzero if in restricted mode */
	char *timedisplay;	/* how time is displayed */
	int beeps;		/* whether beeps are audible */
	char *editmode;		/* input editing mode */
	int tabreply;		/* should replies go into the tab history? */
	int cute;		/* cute messages when a blank line is entered */
	int autodel;		/* automatically delete nicks from the tab
			        history that produce a "not logged on" error */
	char *personalto;	/* 'permanent' personal message recipient */
	int logreplay;		/* should replays go into the log by default? */
	char *alert;		/* alert mode */
	int autoregister;	/* prompt for password when [=Register=] msg
				   received? */
	int asyncread;		/* don't pause output while user is typing */
	int colorize;		/* display colored output */
	int timestamp;		/* display timestamps in messages */
} GLOBS;

/* info on the user's tty */

typedef struct {
	int rows;
	int cols;
} TTYINFO;

/* a linked list of strings */

typedef struct Strlist {
	struct Strlist *next, *prev;
	char str[1];
} STRLIST;

/* misc */

#define CMDCHAR		'/'	/* default char for prefixing commands */
#define PERMCMDCHAR	'\\'	/* like above, but can't be changed */

/* putl (putline) flags */

#define PL_SCR		1	/* text may go to screen */
#define PL_LOG		2	/* text may go to session log */
#define PL_BUF		4	/* text may go to review buffer */
#define PL_TS		8	/* timestamp the event */
#define PL_SL		(PL_SCR | PL_LOG | PL_TS)
#define PL_ALL		(PL_SCR | PL_LOG | PL_BUF | PL_TS)

/* useful defs for modules using TCL code */

/* This is deprecated */
#define TRET	interp->result

/* This is also deprecated */
#define TRETURNERR(x) { Tcl_ResetResult(interp); Tcl_SetResult(interp, x, TCL_VOLATILE); return TCL_ERROR; }


/* ANSI C arg list for C functions that implement Tcl commands
   (string-based) */

#define ARGV_TCL \
	ClientData clientData, Tcl_Interp* interp, int argc, char** argv

/* Macro for returning an error. */

#define RETURN_ERROR(MSG) {						\
	Tcl_AppendResult (interp, argv[0], ": ", (MSG), (char*)NULL);	\
	return TCL_ERROR;						\
}

/* Macro for returning a standard Tcl "wrong # of args" error. */

#define RETURN_ERR_ARGC(ARGMSG) {					\
	Tcl_AppendResult (interp, "wrong # args: should be \"",		\
			  argv[0], " ", (ARGMSG), "\"", (char*)NULL);	\
	return TCL_ERROR;						\
}

/* Cope with pre-object tcl */

#if TCL_MAJOR_VERSION < 8
# define Tcl_GetStringResult(interp) ((interp)->result)
#endif


/* include function prototypes if ANSI compiler */
/* otherwise, include file that shows return type for each function */

#if defined __STDC__ || defined __GNUC__
#include "protos.h"
#else
#include "funcs.h"
#endif

#define MAX_HOSTLEN 256
#define MAX_PASSLEN 256

#define PERSONALSL ".icbservrc"

struct server
{
	char name[MAX_HOSTLEN], host[MAX_HOSTLEN];
	int port;
};

#define COLORSIZE 12	/* how many chars can make an ansi sequence */


struct COLORTABLE {
	char color[COLORSIZE];	/* this color's ansi sequence */
	int defined;		/* has color been set? */ 
};

enum { ColSANE, 
       ColNORMAL, 
       ColNOTICE, 
       ColWARNING, 
       ColERROR, 
       ColNICKNAME,
       ColADDRESS, 
       ColIDLETIME, 
       ColLOGINTIME, 
       ColPERSFROM, 
       ColPERSONAL, 
       ColBEEP, 
       ColMOD, 
       ColUNREG, 
       ColABRKT, 
       ColSBRKT, 
       ColPBRKT, 
       ColWHEAD,
       ColWSUB, 
       ColMORE, 
       ColSTATUS, 
       ColPERSHILITE, 
       ColPERSFROMHILITE,
       ColTIMESTAMP
} colorlist;


/* Returns an ANSI color sequence for 'primary', or for
   'fallback' if no color is associated with 'primary'.
   XXX These should be enum colorlist, not int. */

char* printcolor (int primary, int fallback);

/* Clear the nickname history. */

void histclear (void);

/* Delete a name from the nickname history */

void histdel (char* name);
