/* $Id: send.c,v 1.13 2000/12/13 08:31:56 hoche Exp $ */

/* send various messages to the server */
#include "config.h"

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "icb.h"
#include "externs.h"

#include "../murgil/ipcf.h"
#include "../murgil/murgil.h"
#include "../murgil/externs.h"	/* after ipcf.h */

extern char *findspace();


static void
sendit (char* pkt)
{
	int len = strlen (pkt + 1) + 1;

	if (255 < len) {
		fprintf(stderr,
			"%s[=Error=] Packet too big.%s\n",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	/* Set the packet length */
	*(unsigned char*)pkt = len;

	switch (_sendpacket(port_fd, pkt)) {
	default:
		abort();
	case -2:
		/* lost connection */
		fprintf (stderr, "server connection lost - bye\n");
		exit (1);
	case -1:
		/* error */
		fprintf (stderr, "DEBUG: error in send - panic stop\n");
		exit (1);
	case  0:
		/* incomplete packet */
		fprintf (stderr, "DEBUG: send blocked  - panic stop\n");
		exit (1);
	case  1:
		/* ok */
		break;
	}
}


/* send normal open group message to the server */

void
csendopen(txt)
char *txt;
{
	char **p;
	int i;

	p = msplit(txt, MAX_INPUTSTR);
	if (!p) {
		fprintf(stderr,
			"%s[=Error=] Out of memory for splitting packet.%s",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	for (i=0; p[i]; ++i) {
		/* construct the packet and send it */
		sprintf(pbuf,"%c",M_OPEN);
		strcat(pbuf,p[i]);
		sendit(pp);

		/* and then log it to the buffer and logfile */
		strcpy(mbuf,p[i]);
		putl(mbuf, PL_BUF | PL_LOG);
	}

	msplit_free (p);
}


/* send a login packet */

void
sendlogin(id, nick, group, command, passwd)
char *id, *nick, *group, *command, *passwd;
{

	sprintf(pbuf, "%c%s\001%s\001%s\001%s\001%s",
	  M_LOGIN, id, nick, group, command, passwd);
	sendit(pp);
}

/* send a pong */
void
sendpong()
{
	*pbuf = M_PONG;
	*pp = 1;
	sendit(pp);
}


/* send a command to the server */

void
sendcmd(cmd, args)
char *cmd, *args;
{
	char *nick;
	char **p;
	int i;
	int maxlen;

	if (strlen(cmd) > MAX_INPUTSTR - 5 )
	{
		fprintf(stderr,
			"%s[=Error=] Command too long.%s\n",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	if (!args || *args == '\0')
	{
		sprintf(pbuf, "%c%s\001", M_COMMAND, cmd);
		sendit(pp);
		return;
	}
		
	maxlen = MAX_INPUTSTR - strlen(cmd) - 1;

	/* special case commands that take a nickname */
	if ((strcasecmp(cmd, "write")==0) || (strcasecmp(cmd, "m")==0))
	{
		nick=args;
		args=findspace(args);
		maxlen = maxlen - strlen(nick) - 1;
	}
	else
		nick = NULL;

	p = msplit(args, maxlen);
	if (!p) {
		fprintf(stderr,
			"%s[=Error=] Out of memory to split packet%s",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	for (i=0; p[i]; ++i) {
		sprintf(pbuf,"%c%s\001",M_COMMAND, cmd);
		if (nick) {
			strcat(pbuf, nick);
			strcat(pbuf, " ");
		}
		strcat(pbuf,p[i]);
		sendit(pp);
	}

	msplit_free (p);
}


/* Send a simple command to the icb server, no splitting or
   anything fancy. */

void
send_command (char* cmd, char* arg)
{
	if (MAX_INPUTSTR < strlen (cmd) + 1 + strlen (arg)) {
		fprintf(stderr,
			"%s[=Error=] Command too long.%s\n",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	sprintf (pbuf, "%c%s\001%s", M_COMMAND, cmd, arg);
	sendit (pp);
}


/* Line-split text and send each line as a command. */

void
send_split_command (char* cmd, char* who, char* text)
{
	char** frags;
	int k;
	int n;

	n = MAX_INPUTSTR - strlen (cmd) - 1 - strlen (who) - 1;
	if (n < 0) {
		fprintf(stderr,
			"%s[=Error=] Command too long.%s\n",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	frags = msplit (text, n);
	if (frags == NULL) {
		fprintf(stderr,
			"%s[=Error=] Out of memory to split packet%s",
			printcolor(ColERROR, ColSANE),
			printcolor(ColSANE, ColSANE));
		return;
	}

	for (k = 0; frags[k] != NULL; ++k) {
		sprintf (pbuf, "%c%s\001%s %s", M_COMMAND, cmd, who, frags[k]);
		sendit (pp);
	}

	msplit_free (frags);
}


