/* $Id: signals.c,v 1.22 2000/12/12 07:02:08 hoche Exp $ */

/* signal handling routines */

#include "config.h"

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#include <readline/readline.h>
#include <tcl.h>

#include "icb.h"
#include "externs.h"


/* Note that some of these routines set "continue" to 1. Continue is used */
/* by a few other icb routines to see if they've been interrupted */

/* The signal stuff is not portable, and should be in unix.c. */

#define	mask(s)	(1 << ((s)-1))

static RETSIGTYPE
handle_exit (int sig)
{
    icbexit ();
}

static RETSIGTYPE
handle_intr (int sig)
{
    askquit ();
}


/* handle a stop signal */
/* this technique is something I learned from some other software */
/* someday I need to learn just why it works */

static RETSIGTYPE
handle_stop (int sig)
{
	extern int _rl_meta_flag;  /* private var, sigh. */

	signal(SIGTSTP, SIG_DFL);
#ifndef SOLARIS
	sigsetmask(sigblock(0) &~ mask(SIGTSTP));
#endif

	rl_deprep_term_function ();
	kill(0, SIGTSTP);

	/* XXX this is wrong if we weren't already inside readline, but
           it's only a minor evil.  it will show up if you do ctrl-z during
           a shell escape.  the only real way to fix this is to avoid using
           rl_deprep and handle tty state push/pop on our own. */
	rl_prep_term_function (_rl_meta_flag);
	rl_forced_update_display ();

#ifndef SOLARIS
	sigblock(mask(SIGTSTP));
#endif
	signal(SIGTSTP, handle_stop);
	continued = 1;
}

static RETSIGTYPE
handle_winch (int sig)
{
	getwinsize();
#ifndef HAVE_READLINE_2
	rl_resize_terminal();
#endif
	signal(SIGWINCH, handle_winch);
}

/* set up the signal trappers */

void
trapsignals()
{
	signal (SIGHUP, handle_exit);
	signal (SIGTERM, handle_exit);

	signal (SIGINT, handle_intr);
	signal (SIGQUIT, handle_intr);

	signal (SIGTSTP, handle_stop);
	signal (SIGWINCH, handle_winch);
}


/* verify that the user wants to quit.  note, avoid using stdio in here,
   cuz we may be called from within stdio. */

void
askquit (void)
{
	static char* prompt = "\nReally Quit? ";
	char buf[64];
	int n;

	signal (SIGINT, SIG_IGN);
	write (1, prompt, strlen (prompt));
	n = read (0, buf, sizeof buf);
	if (n <= 0 || buf[0] == 'y' || buf[0] == 'Y') {
		icbexit ();
	}

	signal (SIGINT, handle_intr);
	continued = 1;

	/* XXX shouldn't really do this here, unsafe. */
	rl_forced_update_display();
}

