/* $Id: strings.c,v 1.16 2000/12/13 08:31:57 hoche Exp $ */
#include "config.h"

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "icb.h"
#include "externs.h"

extern char *charmap;

#define OKGROUPCHARS	"-.!'$+,/?_"
#define OKNICKCHARS	"-.!'$+,/?_"

/* replace illegal characters in a nickname */

void
filternickname(txt)
char *txt;
{
	for (; *txt != '\0'; txt++) {
		if ((*txt >= 'A' && *txt <= 'Z') ||
		  (*txt >= 'a' && *txt <= 'z') ||
		  (*txt >= '0' && *txt <= '9'))
			continue;
		if (!strchr(OKNICKCHARS, *txt)) {
			if (*txt == ' ')
				*txt = '_';
			else
				*txt = '?';
		}
	}
}


/* replace illegal characters in a regular line of text */

void
filtertext(s)
char *s;
{
	for (;*s != '\0'; s++)
		if (!(*s >= ' ' && *s < '\177'))
			*s = '_';
}

/* replace illegal characters from a groupname */

void
filtergroupname(txt)
char *txt;
{
	for (; *txt != '\0'; txt++) {
		if ((*txt >= 'A' && *txt <= 'Z') ||
		  (*txt >= 'a' && *txt <= 'z') ||
		  (*txt >= '0' && *txt <= '9'))
			continue;
		if (!strchr(OKGROUPCHARS, *txt)) {
			if (*txt == ' ')
				*txt = '_';
			else
				*txt = '?';
		}
	}
}
/* return 1 if a string is a number */
/* else return 0 */

#include <ctype.h>

/*
numeric(txt)
char *txt;
{
	for (;*txt != '\0'; txt++)
		if (!(*txt >= '0' && *txt <= '9'))
			return(0);
	return(1);
}
*/

char *
findspace(s)
char *s;
{
	/* find EOS or whitespace */
	while (*s != '\0' && *s != ' ' && *s != '\t') s++;

	if (*s == '\0')
		/* oops found no whitespace */
		return(0);
	else {
		/* point to char *after* whitespace */
		*s = '\0';
		return(++s);
	}
}

/* convert a string to lower case */
void
lcaseit(s)
char *s;
{
	for (; *s; s++)
		if (*s >= 'A' && *s <= 'Z')
			 *s |= 040;
}


/* put a string in quotes */
/* puts backslashes before all special chars appearing in the string */
/* doesn't interfere with already backslashed chars */

char *special = "{}[]\";$\\";

void
quoteify(a, b)
char *a, *b;
{
	while (*a != '\0') {
		if (strchr(special, *a)) {
			*b++ = '\\';
			*b++ = *a;
		} else
			*b++ = *a;
		a++;
	}
	*b = '\0';
}


static char *catargsbuf = NULL;

char *
catargs(argv)
char **argv;
{
	char *s, **ap, *p;
	int len = 0;

	for (ap = argv; *ap; ++ap)
	{
		len+=strlen(*ap)+1;
	}
	
	if (catargsbuf)
		free(catargsbuf);

	if ((catargsbuf=(char *)malloc(len+1))==NULL)
	{
		fprintf(stderr,"Out of memory in catargs().\n");
		icbexit();
	}

	p = catargsbuf;
	while (*argv != NULL) {
		s = *argv;
		while (*s) *p++ = *s++;
		if (*(argv+1) != NULL)
			*p++ = ' ';
		argv++;
	}
	*p = '\0';
	return catargsbuf;
}

/* this is a bad hack! */

void
mbreakprint(per, from, s)
int per;
char *from, *s;
{
	int count = 0;
	char *p, *lastw;
	char tmp1, tmp2;

	/* traverse s and try to break on a word */
	p = s;
	lastw = p;
	while (*p != '\0') {
		if (*p == '\n') {
			*p++ = '\0';
			mprint(per, from, s);
			count = 0;
			lastw = p;
			s = p;
			continue;
		}
		if (*p == ' ')
			/* remember location of last word break */
			lastw = p;

		/* break line if we are at max length */
		if (count == (MAX_TEXTLEN - 1)) {
			if ((p - lastw) > 40) {
				/* have to break in the middle of a word */
				tmp1 = *(p - 1);
				tmp2 = *p;
				*(p - 1) = '-';
				*p = '\0';
				mprint(per, from, s);
				*(p - 1) = tmp1;
				*p = tmp2;
				p--;
				s = p;
			} else {
				/* break at last space char */
				tmp1 = *lastw;
				*lastw = '\0';
				mprint(per, from, s);
				*lastw = tmp1;
				p = lastw + 1;
				s = p;
			}
			lastw = p;
			count = 0;
		} else {
			count++;
			p++;
		}
	}
	if (count > 0)
		mprint(per, from, s);
}

void
mprint(per, from, s)
int per;
char *from, *s;
{

/*	In here, we want to test for the existance of from in the tcl
 *	variable personalhilite, and if they exist, printcolor(ColPERSHILITE)
 *	instead.   This means, we should probably have this variable linked
 *	at all times. 
 */
  
	if (per)
	   if ((personalhilite != NULL) &&
	       (strlen((char *)personalhilite)) &&
	       (strcasestr((const char *)personalhilite,(const char *)from)))
		sprintf(mbuf, "%s<*%s%s%s*>%s %s%s%s",
			printcolor(ColPERSFROMHILITE, ColPERSFROM),
			printcolor(ColPERSFROMHILITE, ColPERSFROM),
			from,
			printcolor(ColPERSFROMHILITE, ColPERSFROM),
			printcolor(ColSANE, ColSANE),
			printcolor(ColPERSHILITE, ColPERSONAL),
			s,
			printcolor(ColSANE, ColSANE));
           else
		sprintf(mbuf, "%s<*%s%s%s*>%s %s%s%s",
			printcolor(ColPBRKT, ColPERSFROM),
			printcolor(ColPERSFROM, ColPERSFROM),
			from,
			printcolor(ColPBRKT, ColPERSFROM),
			printcolor(ColSANE, ColSANE),
			printcolor(ColPERSONAL, ColPERSONAL),
			s,
			printcolor(ColSANE, ColSANE));
	else
		sprintf(mbuf, "%s<%s%s%s>%s %s%s%s",
			printcolor(ColABRKT, ColNICKNAME),
			printcolor(ColNICKNAME, ColNICKNAME),
			from,
			printcolor(ColABRKT, ColNICKNAME),
			printcolor(ColSANE, ColSANE),
			printcolor(ColNORMAL, ColNORMAL),
			s,
			printcolor(ColSANE, ColSANE));
	putl(mbuf, PL_ALL);
}

int
wordcmp(s1, s2)
char *s1, *s2;
{
	while(*s1 == *s2++)
		if (*s1 == '\0' || *s1 == ' ' || *s1++ == '\t')
			return(0);
	if (*s1 == ' ' && *(s2-1) == '\0')
		return(0);
	return(*s1 - *--s2);
}

char *
getword(s)
char *s;
{
	static char word[64];
	char *w = word;
	while (*s != ' ' && *s != '\t' && *s != '\0' && ((w - word) < 64))
		*w++ = *s++;
	*w = '\0';
	return(word);
}

/* Read a line containing zero or more colons. Split the string into */
/* an array of strings, and return the number of fields found. */

char *fields[MAX_FIELDS];

int
split(s)
char *s;
{
	char *p = s;
	int i = 0;

	fields[i] = s;
	for(;;) {

		i++;

		/* find delim or EOS */
		while(*p != '\001' && *p != '\0') p++;

		if (*p == '\001') {
			/* got delim */	
			*p = '\0';
			fields[i] = ++p;
		} else
			return(i);
	}
}

char *basename(path)
char *path;
{
char *comp;

comp=path;
while (*path)
{
	if (*path++=='/')
		comp=path;
}

return comp;
}



/* malloc a new string copy of the N chars starting at S,
   possibly adding a '-' if HYPHEN is nonzero */

static char*
msplit_dup (char* s, int n, int hyphen)
{
	char* dup = malloc (n + (hyphen != 0) + 1);
	if (dup != NULL) {
		memcpy (dup, s, n);
		if (hyphen) {
			dup[n++] = '-';
		}
		dup[n] = '\0';
	}
	return dup;
}


/* Splits S for msplit().  Returns the number of fragments,
   or -1 on malloc error.

   Puts malloc'ed copies of the fragments into FRAGS if
   it's non-NULL.  FRAGS must be large enough.  */

static int
msplit_1 (char* s, int size, char** frags)
{
	int nfrags = 0;

	if (s == NULL || size < 1) {
		return 0;
	}

	while (size < strlen (s)) {
		int hyphen = 0;
		char* p = s + size;

		/* find last space */
		while (s < p && ! isspace (*p)) {
			--p;
		}

		/* If no spaces, or space is too early... */
		if (p < s + size / 2) {
			p = s + size - 1;
			hyphen = 1;
		}

		/* Copy it out */
		if (frags) {
			char* frag = msplit_dup (s, p - s, hyphen);
			if (! frag) return -1;
			frags[nfrags] = frag;
		}
		++nfrags;

		/* Skip over the space. */
		s = p + 1 - hyphen;
	}

	/* last fragment */
	if (*s != '\0') {
		if (frags) {
			char* frag = msplit_dup (s, strlen (s), 0);
			if (! frag) return -1;
			frags[nfrags] = frag;
		}
		++nfrags;
	}
	return nfrags;
}

/* Calls free() on a STRINGS array as returned by msplit(). */

void
msplit_free (char** strings)
{
	if (strings != NULL) {
		char ** p = strings;
		for (; *p; ++p) {
			free (*p);
		}
		free (strings);
	}
}


/* Splits STRING into fragments at most SIZE chars each,
   Tries to make the split at a space char (which gets
   deleted).  If no space is available, splits anywhere and
   inserts a '-' at the end of the fragment.

   Returns a NULL-terminated array of the fragments, or NULL
   on malloc error.  The array and all the fragments are
   newly malloc'ed. */

char**
msplit (char* string, int size)
{
	char** frags;
	int nfrags;
	int k;
	
	/* Find out how many fragments we're going to make. */
	nfrags = msplit_1 (string, size, NULL);
	if (nfrags < 0) {
		return NULL;
	}

	/* Allocate the array. */
	frags = (char**) malloc (sizeof (char*) * (nfrags + 1));
	if (frags == NULL) {
		return NULL;
	}

	/* Zero the array. */
	for (k = 0; k < nfrags + 1; ++k) {
		frags[k] = NULL;
	}

	/* Do the split. */
	nfrags = msplit_1 (string, size, frags);
	if (nfrags < 0) {
		msplit_free (frags);
		return NULL;
	}

	return frags;
}
