/* $Id: tcl.c,v 1.17 2000/10/02 11:25:55 hoche Exp $ */

/* TCL interface */
#include "config.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#include "icb.h"
#include "externs.h"

Tcl_Interp *interp;


/* initialize TCL */
void
tclinit()
{
	extern char *defprocs[];
	int x;

	/* initialize the interpreter */
	interp = Tcl_CreateInterp();

	/* add the client primitives */
	Tcl_CreateCommand(interp, "oset", oset, NULL, NULL);
	Tcl_CreateCommand(interp, "ounset", ounset, NULL, NULL);
	Tcl_CreateCommand(interp, "c_log", c_log, NULL, NULL);
	Tcl_CreateCommand(interp, "c_hclear", c_hclear, NULL, NULL);
	Tcl_CreateCommand(interp, "c_hdel", c_hdel, NULL, NULL);
	Tcl_CreateCommand(interp, "c_color", c_color, NULL, NULL);
	Tcl_CreateCommand(interp, "color", c_color, NULL, NULL);
	Tcl_CreateCommand(interp, "c_help", c_help, NULL, NULL);
	Tcl_CreateCommand(interp, "c_hush", c_hush, NULL, NULL);
	Tcl_CreateCommand(interp, "c_quit", c_quit, NULL, NULL);
	Tcl_CreateCommand(interp, "c_rand", c_rand, NULL, NULL);
	Tcl_CreateCommand(interp, "c_replay", c_replay, NULL, NULL);
	Tcl_CreateCommand(interp, "c_shell", c_shell, NULL, NULL);
	Tcl_CreateCommand(interp, "c_time", c_time, NULL, NULL);
	Tcl_CreateCommand(interp, "time", c_time, NULL, NULL);
	Tcl_CreateCommand(interp, "c_usage", c_usage, NULL, NULL);
	Tcl_CreateCommand(interp, "c_version", c_version, NULL, NULL);
	Tcl_CreateCommand(interp, "s_admin", s_admin, NULL, NULL);
	Tcl_CreateCommand(interp, "s_beep", s_beep, NULL, NULL);
	Tcl_CreateCommand(interp, "s_bump", s_bump, NULL, NULL);
	Tcl_CreateCommand(interp, "s_chpw", s_chpw, NULL, NULL);
	Tcl_CreateCommand(interp, "s_motd", s_motd, NULL, NULL);
	Tcl_CreateCommand(interp, "s_group", s_group, NULL, NULL);
	Tcl_CreateCommand(interp, "s_open", s_open, NULL, NULL);
	Tcl_CreateCommand(interp, "s_personal", s_personal, NULL, NULL);
	Tcl_CreateCommand(interp, "s_register", s_register, NULL, NULL);
	Tcl_CreateCommand(interp, "s_send", s_send, NULL, NULL);
	Tcl_CreateCommand(interp, "s_user", s_user, NULL, NULL);
	Tcl_CreateCommand(interp, "s_version", s_version, NULL, NULL);
	Tcl_CreateCommand(interp, "s_who", s_who, NULL, NULL);

	Tcl_LinkVar(interp, "personalhilite", (char *) &personalhilite, TCL_LINK_STRING); 

	/* set up the default bindings */
	for (x = 0; defprocs[x] != 0; x++)
		if (Tcl_Eval(interp, defprocs[x]) != TCL_OK) {
			printf("Default binding failed: %s\r\n", defprocs[x]);
			if (*interp->result != 0)
				puts(interp->result);
		}

}



/* If PATH is an absolute pathname, return it.
   Otherwise try to return $HOME/PATH in a static buffer. */

static char*
home_file (char* path)
{
    static char buf[MAXPATHLEN + 1];
    static char* home = NULL;

    if (! path || *path == '/') {
	return path;
    }

    if (! home) {
	home = getenv ("HOME");
	if (! home) {
	    return path;
	}
    }

    if (strlen (home) + 1 + strlen (path) + 1 < sizeof buf) {
	sprintf (buf, "%s/%s", home, path);
	return buf;
    }

    printf ("Path name too long: %s/%s\r\n", home, path);
    return NULL;
}



static char* icbrc_files[] = {
    ".icbrc",
    ".fnrc",
    ".tclrc",
    NULL,
};

    
static char *
getrcname (void)
{
    char** p;
    char* path;

    path = getenv ("ICBRC");
    if (path == NULL) {
	path = getenv ("FNRC");
    }

    if (path != NULL) {
	return home_file (path);
    }

    for (p = icbrc_files; *p != NULL; ++p) {
	path = home_file (*p);
	if (path && access (path, R_OK) == 0) {
	    return path;
	}
    }
    return NULL;
}

/* Read an icbrc file if it exists.
   If $ICBRC is set, use that.
   If $FNRC is set, use that.  (this is for backward compatibility)
   Otherwise, try ~/.icbrc and a few backward-compatibility names. */

void
tclreadicbrc (void)
{
    char* path = getrcname ();

    if (path == NULL) {
	return;
    }

    printf ("Reading rc file \"%s\"...\r\n", path);
    if (Tcl_EvalFile (interp, path) != TCL_OK) {
	printf ("%s\r\n", Tcl_GetStringResult (interp));
    }
}


void
tclrestrict()
{
	/* delete commands that might shellout or read or write files */
	Tcl_DeleteCommand(interp, "exec");
	Tcl_DeleteCommand(interp, "print");
	Tcl_DeleteCommand(interp, "file");
	Tcl_DeleteCommand(interp, "source");
}
